function onMouseUp() {
    var selectionStart = $('#regex-example').prop("selectionStart");
    var selectionEnd = $('#regex-example').prop("selectionEnd");
    var selection = window.getSelection();
    var selectedText = selection.toString();
    var obj = {};
    obj.startIndex = selectionStart;
    obj.endIndex = selectionEnd;
    // Return control to the textarea if no text is selected to make it editable
    if (selectedText == "")
        return;
    selectText(obj);
}

function selectText(position) {
    $('#regex-example').selectTextInTextArea({
        position: position
    });
}

(function($) {
    let ID = 'select';
    let range = [];
    let SelectTextInTextArea = function($el, config) {
        this.init($el, config);
    };

    SelectTextInTextArea.prototype = {
        init: function($el, config) {
            this.$el = $el;
            this.position = config.position;
            this.generate();
        },

        generate: function() {
            //Adds the backdrop over the textarea to show the selected text
            this.$el
                .addClass(ID + '-input ' + ID + '-content')
                .on('input.' + ID, this.selection.bind(this))
                .on('scroll.' + ID, this.handleScroll.bind(this));

            this.$highlights = $('<div>', { class: ID + '-highlights ' + ID + '-content' });

            this.$backdrop = $('<div>', { class: ID + '-backdrop' })
                .append(this.$highlights);

            this.$container = $('<div>', { class: ID + '-container' })
                .insertAfter(this.$el)
                .append(this.$backdrop, this.$el)
                .on('scroll', this.blockContainerScroll.bind(this));

            this.isGenerated = true;
            this.selection();
        },

        selection: function() {
            range.push([this.position.startIndex, this.position.endIndex])
            let textWithoutTags = this.removeTagsIfPresent(range);
            let boundaries = this.getBoundaries(textWithoutTags);
            this.renderSelection(boundaries);
        },

        removeTagsIfPresent: function(ranges) {
            let textWithoutTags = [];
            ranges.forEach(function(range) {
                let tagsPresent = textWithoutTags.some(function(unstaggeredRange) {
                    let isStartInside = range[0] > unstaggeredRange[0] && range[0] < unstaggeredRange[1];
                    let isStopInside = range[1] > unstaggeredRange[0] && range[1] < unstaggeredRange[1];
                    return isStartInside !== isStopInside;
                });
                if (!tagsPresent) {
                    textWithoutTags.push(range);
                }
            });
            return textWithoutTags;
        },

        getBoundaries: function(ranges) {
            let boundaries = [];
            ranges.forEach(function(range) {
                boundaries.push({
                    type: 'start',
                    index: range[0],
                    className: range.className
                });
                boundaries.push({
                    type: 'stop',
                    index: range[1]
                });
            });

            this.sortBoundaries(boundaries);
            return boundaries;
        },

        sortBoundaries: function(boundaries) {
            boundaries.sort(function(a, b) {
                if (a.index !== b.index) {
                    return b.index - a.index;
                } else if (a.type === 'stop' && b.type === 'start') {
                    return 1;
                } else if (a.type === 'start' && b.type === 'stop') {
                    return -1;
                } else {
                    return 0;
                }
            });
        },

        renderSelection: function(boundaries) {
            let input = this.$el.val();
            boundaries.forEach(function(boundary, index) {
                let markup;
                if (boundary.type === 'start') {
                    markup = '{{select-mark-start|' + index + '}}';
                } else {
                    markup = '{{select-mark-stop}}';
                }
                input = input.slice(0, boundary.index) + markup + input.slice(boundary.index);
            });

            // this keeps scrolling aligned when input ends with a newline
            input = input.replace(/\n(\{\{select-mark-stop\}\})?$/, '\n\n$1');

            // encode HTML entities
            input = input.replace(/</g, '&lt;').replace(/>/g, '&gt;');

            if (this.browser === 'ie') {
                // IE/Edge wraps whitespace differently in a div vs textarea, this fixes it
                input = input.replace(/ /g, ' <wbr>');
            }

            // replace start tokens with opening <mark> tags with class name
            input = input.replace(/\{\{select-mark-start\|(\d+)\}\}/g, function(match, submatch) {
                var className = boundaries[+submatch].className;
                if (className) {
                    return '<mark class="' + className + '">';
                } else {
                    return '<mark>';
                }
            });

            // replace stop tokens with closing </mark> tags
            input = input.replace(/\{\{select-mark-stop\}\}/g, '</mark>');

            this.$highlights.html(input);
        },

        handleScroll: function() {
            let scrollTop = this.$el.scrollTop();
            this.$backdrop.scrollTop(scrollTop);

            // For Chrome and Safari
            let scrollLeft = this.$el.scrollLeft();
            this.$backdrop.css('transform', (scrollLeft > 0) ? 'translateX(' + -scrollLeft + 'px)' : '');
        },

        // For Chrome
        blockContainerScroll: function() {
            this.$container.scrollLeft(0);
        },

        destroy: function() {
            this.$backdrop.remove();
            this.$el
                .unwrap()
                .removeClass(ID + '-text ' + ID + '-input')
                .off(ID)
                .removeData(ID);
        },
    };

    // register the jQuery plugin
    $.fn.selectTextInTextArea = function(position) {
        return this.each(function() {
            let $this = $(this);
            let plugin = $this.data(ID);
            if (plugin) {
                plugin.destroy();
            }
            plugin = new SelectTextInTextArea($this, position);
            if (plugin.isGenerated) {
                $this.data(ID, plugin);
            }
        });
    };
})(jQuery);